local AceAddon = LibStub("AceAddon-3.0")

SbHealthstone = AceAddon:NewAddon("SbHealthstone", "AceTimer-3.0", "AceEvent-3.0")

local SbHealthstone, self = SbHealthstone, SbHealthstone

self.db = nil

local L = LibStub("AceLocale-3.0"):GetLocale("SbHealthstone")

self.obj = LibStub("LibDataBroker-1.1"):NewDataObject("Healthstone", {
    type = "data source", 
    icon = "Interface\\Icons\\INV_Stone_04", 
    OnClick = function(_, msg)
			if msg == "LeftButton" then
				self:RecalcText()
			elseif msg == "RightButton" then
				if LibStub("AceConfigDialog-3.0").OpenFrames["Statblock_Healthstone"] then
					LibStub("AceConfigDialog-3.0"):Close("Statblock_Healthstone")
				else
					self:ShowConfig()
				end
			end
		end,
    text = 0
  })

local DEFAULT_RANK = 6

local function getWatchListDefaults()
	local result = {}
	for rank, t in ipairs(self.healthstones) do
		result["rank" .. rank] = {}
		for i,v in pairs(t) do
			result["rank" .. rank][v.hp .. "hp"] = (rank == DEFAULT_RANK)
		end
	end
	return result
end

function SbHealthstone:OnInitialize()
	local defaults = {
		["profile"] = {
			["textoptions"] = {
				["style"] = "SPvalues",
				["delimiter"] = "/",
				["rankdelimiter"] = ":",
			},
			["watchlist"] = getWatchListDefaults()
		},
	}
	
	self.db = LibStub("AceDB-3.0"):New("SbHealthstoneDB", defaults)
	self:RegisterEvent("BAG_UPDATE", "RecalcText")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", function()
	  self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	  self:RecalcText()
	end)
-- 	self:SetupFuBar()
	self:InitOptions()
end



-- function SbHealthstone:SetupFuBar()
-- 
-- 	if LibStub:GetLibrary("LibFuBarPlugin-Mod-3.0") then
-- 		self:SetFuBarText("SbHealthstone")
-- 		-- self:SetFuBarOption('tooltipType', "Tablet-2.0")
-- 		
-- 		self:SetFuBarOption('hasIcon', true)
-- 		self:SetFuBarOption('iconPath', [[Interface/Icons/INV_Stone_04]])
-- 	end
-- 	
-- end

-- item-ids
self.healthstones = {
	-- Minor Healthstone
	[1] = {
		[0] = { ["itemid"] = 5512, ["hp"] = 100, },
		[1] = { ["itemid"] = 19004, ["hp"] = 110, },
		[2] = { ["itemid"] = 19005, ["hp"] = 120, },
	},
	-- Lesser Healthstone
	[2] = {
		[0] = { ["itemid"] = 5511, ["hp"] = 250, },
		[1] = { ["itemid"] = 19006, ["hp"] = 275, },
		[2] = { ["itemid"] = 19007, ["hp"] = 300, },
	},
	[3] = {
		[0] = { ["itemid"] = 5509, ["hp"] = 500, },
		[1] = { ["itemid"] = 19008, ["hp"] = 550, },
		[2] = { ["itemid"] = 19009, ["hp"] = 600, },
	},
	[4] = {
		[0] = { ["itemid"] = 5510, ["hp"] = 800, },
		[1] = { ["itemid"] = 19010, ["hp"] = 880, },
		[2] = { ["itemid"] = 19011, ["hp"] = 960, },
	},
	[5] = {
		[0] = { ["itemid"] = 9421, ["hp"] = 1200, },
		[1] = { ["itemid"] = 19012, ["hp"] = 1320, },
		[2] = { ["itemid"] = 19013, ["hp"] = 1440, },
	},
	[6] = {
		[0] = { ["itemid"] = 22103, ["hp"] = 2080, },
		[1] = { ["itemid"] = 22104, ["hp"] = 2288, },
		[2] = { ["itemid"] = 22105, ["hp"] = 2496, },
	},
}

local function round(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) / mult
end


local function getShortString(number)
	if number < 1000 then
		return number
	else
		return round(number/1000,1)
	end
end

local function hpvaluefunc(delimiter, rankdelimiter)
	local text = " "
	for rank,t in ipairs(self.healthstones) do
		local empty = true
		for i,v in pairs(t) do
			
			if self.db.profile.watchlist["rank" .. rank][v.hp .. "hp"] then
				
				if GetItemCount(v.itemid) > 0 then
					text = text .. "|cff00ff00" .. v.hp .. "|r"
				else
					text = text .. "|cffff0000" .. v.hp .. "|r"
				end
				text = text .. delimiter
				empty = false
			end
		end
		
		if not empty then
			text = text:sub(0, -(delimiter:len() + 1))
			text = text .. rankdelimiter
		end
	end
	text = text:sub(0, -(rankdelimiter:len() + 1))
	return text
end

local function hpvalueshortfunc(delimiter, rankdelimiter)
	local text = ""
	for rank,t in pairs(self.healthstones) do
		local empty = true
		for k,v in pairs(t) do
			
			if self.db.profile.watchlist["rank" .. rank][v.hp .. "hp"] then
				
				if GetItemCount(v.itemid) > 0 then
					text = text .. "|cff00ff00" .. getShortString(v.hp) .. "|r"
				else
					text = text .. "|cffff0000" .. getShortString(v.hp) .. "|r"
				end
				text = text .. delimiter
				empty = false
			end
		end
		
		if not empty then
			text = text:sub(0, -(delimiter:len() + 1))
			text = text .. rankdelimiter
		end
	end
	text = text:sub(0, -(rankdelimiter:len() + 1))
	return text
end

local function spvaluefunc(delimiter, rankdelimiter)
	local text = ""
	
	for rank,t in ipairs(self.healthstones) do
		local empty = true
		
		for k,v in pairs(t) do
			
			if self.db.profile.watchlist["rank" .. rank][v.hp .. "hp"] then
				
				if GetItemCount(v.itemid) > 0 then
					text = text .. "|cff00ff00" .. k .. "|r"
				else
					text = text .. "|cffff0000" .. k .. "|r"
				end
				text = text .. delimiter
				empty = false
			end
		end
		
		if not empty then
			text = text:sub(0, -(delimiter:len() + 1))
			text = text .. rankdelimiter
		end
	end
	text = text:sub(0, -(rankdelimiter:len() + 1))
	return text
	
end

local function symbolsfunc(delimiter, rankdelimiter)
	local text = ""
	for rank,t in ipairs(self.healthstones) do
		local empty = true
		for i,v in pairs(t) do
			
			if self.db.profile.watchlist["rank" .. rank][v.hp .. "hp"] then
				
				if GetItemCount(v.itemid) > 0 then
					text = text .. "|cff00ff00+|r"
				else
					text = text .. "|cffff0000-|r"
				end
				text = text .. delimiter
				empty = false
			end
		end
		
		if not empty then
			text = text:sub(0, -(delimiter:len() + 1))
			text = text .. rankdelimiter
		end
	end
	text = text:sub(0, -(rankdelimiter:len() + 1))
	return text
end

local function hpvalueListfunc(delimiter, rankdelimiter)
	
	local text = ""
	for rank,t in ipairs(self.healthstones) do
		local empty = true
		for i,v in pairs(t) do
			
			if self.db.profile.watchlist["rank" .. rank][v.hp .. "hp"] then
				
				if GetItemCount(v.itemid) > 0 then
					text = text .. "|cff00ff00" .. v.hp .. "|r"
					text = text .. delimiter
					empty = false
				end
				
			end
		end
		
		if not empty then
			text = text:sub(0, -(delimiter:len() + 1))
			text = text .. rankdelimiter
		end
	end
	text = text:sub(0, -(rankdelimiter:len() + 1))
	
	if text == "" then
		return "|cffff0000None|r"
	end
	
	return text
	
end

local function hpvalueListshortfunc(delimiter, rankdelimiter)
	
	local text = ""
	for rank,t in ipairs(self.healthstones) do
		local empty = true
		for i,v in pairs(t) do
			
			if self.db.profile.watchlist["rank" .. rank][v.hp .. "hp"] then
				
				if GetItemCount(v.itemid) > 0 then
					text = text .. "|cff00ff00" .. getShortString(v.hp) .. "|r"
					text = text .. delimiter
					empty = false
				end
				
			end
		end
		
		if not empty then
			text = text:sub(0, -(delimiter:len() + 1))
			text = text .. rankdelimiter
		end
	end
	text = text:sub(0, -(rankdelimiter:len() + 1))
	
	if text == "" then
		return "|cffff0000None|r"
	end
	
	return text
	
end


local displayStyles = {
	["HPvalues"] = hpvaluefunc,
	["HPvaluesShort"] = hpvalueshortfunc,
	["SPvalues"] = spvaluefunc,
	["symbols1"] = symbolsfunc,
	["HPvaluesList"] = hpvalueListfunc,
	["HPvaluesListShort"] = hpvalueListshortfunc,
}

local function update()

		local displayfunc = displayStyles[self.db.profile.textoptions.style]
		local delimiter = self.db.profile.textoptions.delimiter
		local rankdelimiter = self.db.profile.textoptions.rankdelimiter
		self.obj.text = displayfunc(delimiter, rankdelimiter)
end

function SbHealthstone:RecalcText()
	update()
end