local SbHealthstone, self = SbHealthstone, SbHealthstone

local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("SbHealthstone")


local function getWatchListOptions()
	local result = {}
	for rank, t in ipairs(self.healthstones) do
		result["rank" .. rank] = {
			type = "group",
			name = "Rank " .. rank,
			desc = "Rank " .. rank .. " Healthstones",
			args = {},
		}
		for i,v in pairs(t) do

			result["rank" .. rank].args[v.hp .. "hp"] = {
				type = "toggle",
				name = v.hp .. "HP",
				desc = v.hp .. "HP",
				get = function() return self.db.profile.watchlist["rank" .. rank][v.hp .. "hp"] end,
				set = function(info, val) 
					self.db.profile.watchlist["rank" .. rank][v.hp .. "hp"] = val 
					self:RecalcText()
				end,
			}
			
		end
		
	end
	
	return result
	
end

local function getOptions()
	if not self.options then
		self.options = {
			name = "Statblock_Healthstone",
			desc = L["Information about healthstones in your inventory"],
			type = 'group',
			args = {
				textoptions = {
					type = "group",
					name = L["Text options"],
					desc = L["Options for the text"],
					args = {
						style = {
							type = "select",
							name = L["Textstyle"],
							desc = L["Style of the text"],
							values = {
								HPvalues = "HPvalues", 
								HPvaluesShort = "HPvalues (short)", 
								SPvalues = "SPvalues", 
								symbols1 = "Symbols",
								HPvaluesList = "HPvalues - List",
								HPvaluesListShort = "HPvalues (Short) - List",
							},
							get = function() return self.db.profile.textoptions.style end,
							set = function(info, val) 
								self.db.profile.textoptions.style = val
								self:RecalcText()
							end,
						},
						delimiter = {
							type = "input",
							name = L["Delimiter"],
							desc = L["Delimiter between the different stonetypes"],
							get = function() return self.db.profile.textoptions.delimiter end,
							set = function(info, val)
								self.db.profile.textoptions.delimiter = val
								self:RecalcText()
							end
						},
						rankdelimiter = {
							type = "input",
							name = L["Rank-delimiter"],
							desc = L["Delimiter between different ranks of stones"],
							get = function() return self.db.profile.textoptions.rankdelimiter end,
							set = function(info, val)
								self.db.profile.textoptions.rankdelimiter = val
								self:RecalcText()
							end,
						},
					},
				},
				watchlist = {
					type = "group",
					name = L["Watchlist"],
					desc = L["Healthstones to be displayed"],
					args = getWatchListOptions(),
				},
				profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db),
			}
		}
	end
	

	return self.options
end


function SbHealthstone:ShowConfig()
	AceConfigDialog:SetDefaultSize("Statblock_Healthstone", 500, 300)
	AceConfigDialog:Open("Statblock_Healthstone")
end

function SbHealthstone:InitOptions()
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Statblock_Healthstone", getOptions, "healthstonefu")
	self.OnMenuRequest = self.options
	self.OpenMenu = self.ShowConfig
	AceConfigDialog:AddToBlizOptions("Statblock_Healthstone", "Statblock_Healthstone")
	
end